# -*- coding: utf-8 -*-
import os
import io
import json
from ruamel.yaml import round_trip_dump, round_trip_load, load, dump_all
from bson.json_util import (default as bson_object_default,
                            object_hook as bson_object_hook)


def dump_yaml_file(data, path, round_tripping=False):
    with io.open(path, 'w', encoding='utf-8') as writer:
        if round_tripping:
            round_trip_dump(data, writer, allow_unicode=True, )
        else:
            dump_all([data], writer, allow_unicode=True)


def load_yaml_file(path, round_tripping=False):
    data = None
    with io.open(path, 'r', encoding='utf-8') as reader:
        if round_tripping:
            data = round_trip_load(reader)
        else:
            data = load(reader)
    return data


def mkdir(path):
    """

    :param path:
    :return:
    """
    path = path.strip()
    path = path.rstrip("\\")

    is_exists = os.path.exists(path)
    if not is_exists:
        os.makedirs(path)
    else:
        pass


def to_json(obj):
    return json.dumps(obj, ensure_ascii=False, sort_keys=True, default=bson_object_default, indent=4,
                      encoding='utf-8')


# -*- coding:utf-8 -*-
import os
import sys
import glob
import copy
import random
import string
import urllib
import json
from collections import OrderedDict
import logging
from . import utils
from . import datetimeutil
from . import exception as errors
from lcyframe.libs import yaml2py
from lcyframe.libs.utils import fix_path

class AnanasRestApi(object):

    content = ""

    def __init__(self, **kwargs):
        """
        加载配置文件
        """
        self.config = kwargs.get('api_doc_config')
        self.path = kwargs.get('path')
        self.schema_template = self.config.get("schema_template") or "default"
        self.api_schema_mp = {
            "lcyframe": self.process_resource_context,
        }
    def set_api(self):
        """

        :return:
        """
        self.content += "## " + self.config.get('title') + "\n\n"
        self.set_yml_md()
        self.make_md()

    def set_yml_md(self):
        """

        :return:
        """
        path = self.config.get('api_schema_dir')
        leve_str = str(self.config.get('leve')) + "."
        leve = 1

        if self.schema_template != "lcyframe":
            self.definitions = yaml2py.load_yaml_file(os.path.join(path, "definitions.yml"))
        else:
            self.definitions = {}

        try:
            if not os.path.exists(path):
                raise Exception("The docs dir not exists %s" % path)

            l = glob.glob('%s/*' % path)
            for pkg_dir in l[::-1]:
                if os.path.basename(pkg_dir) == "definitions.yml":
                    continue

                leve, leve_str = self.generate_content(pkg_dir, leve, leve_str)
        except errors.AnanasDocError as e:
            print("[Error]:" + str(e.code) + ":" + e.message)
            exit()

    def generate_content(self, path, leve, leve_str):
        """
        :param path:
        :return:
        """
        if os.path.isdir(path):
            for p in os.listdir(path):
                return self.generate_content(fix_path(os.path.join(path, p)), leve, leve_str)
        else:
            if path.split(".")[-1] != "yml":
                logging.warning("the %s is not .yml format." % path)
                raise Exception("the %s is not .yml format." % path)

            resource_text = yaml2py.load_yaml_file(path, False)
            _leve_str = leve_str + str(leve)

            if not resource_text:
                return

            if "model" not in resource_text[0]:
                logging.warning("require key 'model' in %s " % path)

            model_name = resource_text[0]["model"] or "未命名的模块"
            self.content += "### " + _leve_str + "、 " + model_name + "\n"

            n = 1
            for resource_def in resource_text:
                # _leve_str = leve_str + str(leve)
                self.api_schema_mp[self.schema_template](resource_def, _leve_str, n)
                n += 1

            leve += 1

        return leve, leve_str

    def process_resource_context(self, resource_def, _leve_str, n):
        """

        :param resource_def:
        :param leve_str:
        :return:
        """
        uri = resource_def["apis"]
        for method, data_mp in resource_def['method'].items():
            self.content += "#### " + _leve_str + "." + str(n) + "、"
            self.content += (data_mp.get('summary', {}) or "功能名称") + "\n"
            if data_mp.get('description', ""):
                self.content += "**说明：%s**\n\n" % str(data_mp.get('description', "") or "方法说明")
            self.content += "**接口：**<strong><font color='#e48928' size='4px'>%s</font></strong>\n\n" % uri
            if method.upper() == "GET":
                color_value = "#2eae21"
            elif method.upper() == "POST":
                color_value = "#ec899a"
            elif method.upper() == "PUT":
                color_value = "#3de485"
            else:
                color_value = "#eb1c44"
            self.content += "**方法：**<font color='%s'><b>%s</b></font>\n\n" % (color_value, method.upper())
            self.content += "**参数：**\n\n"
            self.content += "|参数|类型|必须|允许值|描述|\n"
            self.content += "|:--------|:--------|:------|:--------|:------|\n"

            # request params
            parameters = data_mp.get('parameters', [])
            if not parameters:
                self.content += "|_|_|_|_|_|\n"
            self.set_req_md(copy.deepcopy(parameters))

            # response params
            self.content += "\n**成功响应：**\n"
            self.content += "\n```python"
            self.content += "\n"

            responses = data_mp.get('responses', None)
            if responses:
                self.set_resp_md(copy.deepcopy(responses))
            else:
                self.content += "{\n\n}\n"

            self.content += "\n```\n"

            # reference
            # reference = data_mp.get("reference", "")
            # self.set_reference_md(reference)

    def generate_content2(self, path, leve, leve_str):
        """
        风格2：文档作出导航，按照以下三级节点生成
        模块：
            handler
                get
                post
                delete
                put
        :param path:
        :param leve:
        :param leve_str:
        :return:
        """
        if os.path.isdir(path):
            for p in os.listdir(path):
                return self.generate_content(fix_path(os.path.join(path, p)), leve, leve_str)
        else:
            if path.split(".")[-1] != "yml":
                logging.warning("the %s is not .yml format." % path)
                raise Exception("the %s is not .yml format." % path)

            resource_text = yaml2py.load_yaml_file(path, False)
            _leve_str = leve_str + str(leve)

            if not resource_text:
                return

            if "model" not in resource_text[0]:
                logging.warning("require key 'model' in %s " % path)
            self.content += "### " + _leve_str + "、 " + resource_text[0]["model"] + "\n"

            n = 1
            for resource_def in resource_text:
                # _leve_str = leve_str + str(leve)
                self.api_schema_mp[self.schema_template](resource_def, _leve_str, n)
                n += 1

            leve += 1

        return leve, leve_str

    def process_resource_context2(self, resource_def, _leve_str, n):
        """

        :param resource_def:
        :param leve_str:
        :return:
        """
        uri = resource_def["apis"]
        name = resource_def.get('name', "") or "未命名的模块"
        self.content += "#### " + _leve_str + "." + str(n) + "、" + name + "\n"

        description = resource_def.get('description')
        if description:
            self.content += "**说明：**" + description + "\n"

        leve = 1
        for method, data_mp in resource_def['method'].items():
            self.content += "##### " + _leve_str + "." + str(n) + "." + str(leve) + "、"
            self.content += (data_mp.get('summary', {}) or "功能名称") + "\n"
            if data_mp.get('description', ""):
                self.content += "**说明：%s**\n\n" % str(data_mp.get('description', "") or "方法说明")
            self.content += "**接口：**<strong><font color='#e48928' size='4px'>%s</font></strong>\n\n" % uri
            if method.upper() == "GET":
                color_value = "#2eae21"
            elif method.upper() == "POST":
                color_value = "#ec899a"
            elif method.upper() == "PUT":
                color_value = "#3de485"
            else:
                color_value = "#eb1c44"
            self.content += "**方法：**<font color='%s'><b>%s</b></font>\n\n" % (color_value, method.upper())
            self.content += "**参数：**\n\n"
            self.content += "|参数|类型|必须|允许值|描述|\n"
            self.content += "|:--------|:--------|:------|:--------|:------|\n"

            # request params
            parameters = data_mp.get('parameters', [])
            if not parameters:
                self.content += "|_|_|_|_|_|_|\n"
            self.set_req_md(copy.deepcopy(parameters))

            # response params
            self.content += "\n**成功响应：**\n"
            self.content += "\n```json"
            self.content += "\n"

            responses = data_mp.get('responses', None)
            if responses:
                self.set_resp_md(copy.deepcopy(responses))
            else:
                self.content += "{\n\n}\n"

            self.content += "\n```\n"

            leve += 1

    def set_req_md(self, rules):
        """

        :return:
        """
        if type(rules) == list:
            for rule in rules:
                self.set_req_params_md(rule)
                if rule['type'] == 'dict':
                    if rule.get('schema'):
                        self.set_req_md(rule)
        if type(rules) == dict:
            if rules['type'] == 'dict':
                for rule in rules['schema']:
                    rule['name'] = "{" + rules['name'] + "}." + rule.get('name', '')
                    self.set_req_params_md(rule)

    def set_req_params_md(self, rule):
        """

        :param content:
        :return:
        """
        required = rule.get('required', None)
        if required is True:
            rule['required'] = "是"
        else:
            rule['required'] = "否"

        if not rule.get('description'):
            rule["description"] = "无说明"

        description = str(rule['description']).replace('|', '、')

        en_value_str = ""
        if rule.get('allow', []):
            en_value_str = ", ".join(map(str, rule["allow"]))
        if rule.get('allowed', []):
            en_value_str = ", ".join(map(str, rule["allowed"]))

        self.content += "|" + '<font color="#9d127b"><strong>' + rule['name'] + '</strong></font>' + \
                        "|" + rule['type'] + \
                        "|" + rule['required'] + \
                        "|" + en_value_str + \
                        "|" + str(description) + \
                        "|\n"       # "|" + rule.get("in", "query") + \

    def set_resp_md(self, responses):
        """
        响应数据组装
        :return:
        """
        self.space_seq = 1

        def get_space():
            return self.space_seq * "   "

        def get_type_fun(obj):
            t = type(obj)
            if t in [dict, list, tuple]:
                return {
                    dict: gen_dict_obj,
                    list: gen_list_obj,
                    tuple: gen_list_obj,
                }[t](obj)
            else:
                return get_string_obj(obj)

        def get_string_obj(obj):
            STR2TYPE = {"int": int,
                        "integer": int,
                        "string": str,
                        "str": str,
                        "unicode": "",
                        "float": float,
                        "bool": bool,
                        "dict": dict,
                        "json": ""}
            suffix = ""
            for t, v in STR2TYPE.items():
                if "|%s" % t in str(obj):
                    # 代码模式下，冲突的字符导致样式失效，需要替换
                    if type(obj) in [str, bytes] and "|" in obj:
                        if "，" in obj:
                            obj = obj.replace("，", ",")
                        if "：" in obj:
                            obj = obj.replace("：", ":")
                        if '"' in obj:
                            obj = obj.replace('"', "'")
                        if '（' in obj:
                            obj = obj.replace('（', "(")
                        if '）' in obj:
                            obj = obj.replace('）', ")")
                        if '；' in obj:
                            obj = obj.replace('；', ";")

                    obj = obj.replace("|%s" % t, "")
                    suffix = ", 类型%s" % t

            # if type(obj) in [str, unicode]:
            #     obj = "\"" + obj + "\""
            #
            #     obj = obj + suffix

            # 1、字符串类型的数字、文字，需要加双引号
            # 2、整形，浮点型，bool类型的值不需要
            s = ""
            if type(obj) in [str, bytes] and "类型" not in suffix:
                s += '"%s"' % (str(obj) + suffix)
            else:
                s += str(obj) + suffix
            return s + ",\n"

        def gen_dict_obj(obj):
            self.space_seq += 1
            space = get_space()

            str = ""
            # str += "\n"
            str += "{\n"
            for k, n in obj.items():
                str += space + "  \"%s\"" % k + ": " + get_type_fun(n)
            str += space + "},\n"
            self.space_seq -= 1
            return str

        def gen_list_obj(obj):
            self.space_seq += 1
            space = get_space()

            str = ""
            # str += "\n"
            str += "["
            for item in obj:
                str += "\n"
                str += space + "   " + get_type_fun(item)
                str += space + "   ...,\n"
                break

            str += space + "],"
            str += "\n"
            self.space_seq -= 1
            return str

        self.content += "{\n"

        for key, value in responses.items():
            self.content += get_space()
            self.content += "\"%s\"" % key + ": " + get_type_fun(value)
            self.space_seq = 1

        self.content += "}" if responses else "\n}"

    def set_reference_md(self, reference):
        """
        组装参考描述
        :param reference:
        :return:
        """
        if not reference:
            return

        self.content += "**参考说明：**\n"
        self.content += "```dart\n"
        for index, s in enumerate(reference.split("\\")):
            self.content += str(index+1)
            self.content += s + "\n"

        self.content += "```\n"

    def make_md(self):
        """

        :return:
        """

        f = open(self.path + '/docs/api.md', "w+", encoding='utf-8')
        f.write(self.content)
        f.close()




if __name__ == "__main__":
    kw = {
        "api_doc_config": {
            "if_set_api": True,
            "api_schema_dir": "/Users/Song/Desktop/www/docs/api_schema",
            "leve": 2,
            "title": "接口文档",
            "schema_template": "lcylln"
        },
        'path': "/Users/Song/Desktop/www/docs/doc",

    }
    a = AnanasRestApi(**kw)
    a.set_api()


#-*- coding:utf-8 -*-
import calendar
from functools import partial
import time
from datetime import datetime, date, timedelta

import pytz

utc = pytz.utc

PRC_TZ_STR = 'Asia/Shanghai'
prc = pytz.timezone(PRC_TZ_STR)


def is_aware(value):
    """
    Determines if a given datetime.datetime is aware.

    The logic is described in Python's docs:
    http://docs.python.org/library/datetime.html#datetime.tzinfo
    """
    return value.tzinfo is not None and value.tzinfo.utcoffset(value) is not None


def is_naive(value):
    """
    Determines if a given datetime.datetime is naive.

    The logic is described in Python's docs:
    http://docs.python.org/library/datetime.html#datetime.tzinfo
    """
    return value.tzinfo is None or value.tzinfo.utcoffset(value) is None


def make_aware(value, timezone):
    """
    Makes a naive datetime.datetime in a given time zone aware.
    """
    if hasattr(timezone, 'localize'):
        # available for pytz time zones
        return timezone.localize(value, is_dst=None)
    else:
        # may be wrong around DST changes
        return value.replace(tzinfo=timezone)


def make_naive(value, timezone):
    """
    Makes an aware datetime.datetime naive in a given time zone.
    """
    value = value.astimezone(timezone)
    if hasattr(timezone, 'normalize'):
        # available for pytz time zones
        value = timezone.normalize(value)
    return value.replace(tzinfo=None)


def make_prc_datetime(*args, **kwargs):
    d = datetime(*args, **kwargs)
    return make_aware(d, prc)


def now(timezone=utc):
    """
    Returns an aware datetime.datetime
    """
    return make_aware(datetime.utcnow(), timezone)


utcnow = datetime.utcnow


def utc2prc(value):
    """
    Convert utc to PRC local datetime
    """
    if is_naive(value):
        value = make_aware(value, utc)
    return value.astimezone(prc)


def format_prc(value, fmt=None):
    if isinstance(value, datetime):
        if fmt is None:
            fmt = '%Y-%m-%d %H:%M:%S'
        return utc2prc(value).astimezone(prc).strftime(fmt)
    return value


def format_utc(value, fmt=None):
    if fmt is None:
        fmt = '%Y-%m-%d %H:%M:%S UTC'
    return value.strftime(fmt)


def prc_now(fmt='%Y-%m-%d %H:%M:%S'):
    return utc2prc(datetime.utcnow()).strftime(fmt)


def parse_utc_from_str(time_str, zone_name, fmt='%Y-%m-%d %H:%M:%S'):
    """
    Parse a datetime with given zone and convert to utc datetime
    :param time_str:
    :type time_str:
    :param zone_name:
    :type zone_name:
    :param fmt:
    :type fmt:
    :return:
    :rtype:
    """
    zone = pytz.timezone(zone_name)
    # assume naive
    t = datetime.strptime(time_str, fmt)
    # convert to aware
    t2 = make_aware(t, zone)
    return make_naive(t2, utc)


def parse_prc_from_day_str(day_str, fmt='%Y%m%d'):
    """
    解析日期形式的字符串（PRC本地时间）并转换到UTC navie datetime
    :param day_str:
    :param fmt: default is '%Y%m%d'
    :return: datetime UTC naive
    """
    return parse_utc_from_str(day_str, PRC_TZ_STR, fmt)


def datetime2timestamp(datetime_=None):
    """
    Convert local datetime to unix timestamp
    """
    if datetime_ is None:
        datetime_ = datetime.now()
    if not isinstance(datetime_, datetime):
        return 0
    return datetime_ and int(time.mktime(datetime_.timetuple()))


def utc2timestamp(utc_datetime=None):
    """
    Convert utc datetime to seconds since epoch (UTC)
    """
    if utc_datetime is None:
        return int(time.time())
    return calendar.timegm(utc_datetime.utctimetuple())


MINUTE = 60
HOUR = MINUTE * 60
DAY = HOUR * 24


def expired_ttl(days=0, hours=0, minutes=0, seconds=0, current=None):
    """
    Compute a expired timestamp
    """
    if not current:
        current = int(time.time())
    return current + days * DAY + hours*HOUR + minutes * MINUTE + seconds


def next_time(from_now=None, **kwargs):
    if from_now is None:
        from_now = datetime.utcnow()
    _next_time = from_now + timedelta(**kwargs)
    return _next_time


def after_days(days=1):
    return next_time(days=days).date()


def date2int(date_v):
    """
    >>> date2int(datetime.date(2013,11,1))
    20131101
    :param date_v:
    :type date_v:
    :return:
    :rtype:
    """
    return int(date_v.strftime('%Y%m%d'))


def make_date_timestamp(year, month, day):
    d = datetime(year, month, day, 23, 59, 59)
    return calendar.timegm(d.utctimetuple())


def make_date(year, month, day):
    try:
        return date(year, month, day)
    except ValueError:
        return

_all_chunks = (
    (60 * 60 * 24 * 365, u'年'),
    (60 * 60 * 24 * 30, u'月'),
    (60 * 60 * 24 * 7, u'周'),
    (60 * 60 * 24, u'天'),
    (60 * 60, u'小时'),
    (60, u'分钟')
)

_day_chunks = (
    (60 * 60 * 24, u'天'),
    (60 * 60, u'小时'),
    (60, u'分钟')
)


def compute_relative_time(timestamp=1340875790, ago=True, chunks=None):
    #git/data.c void show_date_relative
    delta = int(time.time()) - int(timestamp)
    # In unitests we have 0 or 1 seconds, not stable,
    # better say 'few' when below 10 s.
    if delta < 10:
        return u"刚刚"
    if delta < 90:
        return unicode(delta) + u" 秒前" if ago else unicode(delta)

    if chunks is None:
        chunks = _all_chunks

    for i, (seconds, name) in enumerate(chunks):
        count = delta // seconds
        if count != 0:
            break

    s = u'{0} {1}'.format(count, name)

    if i + 1 < len(chunks):
        seconds2, name2 = chunks[i + 1]
        count2 = (delta - (seconds * count)) // seconds2
        if count2 != 0:
            s += u' {0} {1}'.format(count2, name2)
    return s + u' 以前' if ago else s


compute_relative_days = partial(compute_relative_time, chunks=_day_chunks)


# def current_month_range_datetime():
#     """
#     返回当前月的start/end datetime(next month first time)
#     :return: list
#     """
#     _now = utcnow()
#     first_time = datetime(_now.year, _now.month, 1, 0, 0, 0, 0)
#     last_time = first_time + relativedelta.relativedelta(months=1)
#     return first_time, last_time
#
#
# def today_range_datetime():
#     _now = utcnow()
#     first_time = datetime(_now.year, _now.month, _now.day, 0, 0, 0, 0)
#     last_time = first_time + relativedelta.relativedelta(days=1)
#     return first_time, last_time


# def before_month_range_datetime(months=1):
#     """
#     返回前一个月的start/end datetime(current month first time)
#     :return: list
#     """
#     _now = utcnow()
#     last_time = datetime(_now.year, _now.month, 1, 0, 0, 0, 0)
#     first_time = last_time - relativedelta.relativedelta(months=months)
#     return first_time, last_time


# def this_month_range_datetime(month, year=None):
#     _now = utcnow()
#     if year is None:
#         year = _now.year
#     first_time = datetime(year, month, 1, 0, 0, 0, 0)
#     last_time = first_time + relativedelta.relativedelta(months=1)
#     return first_time, last_time


def remain_days(target_datetime, from_datetime=None):
    if from_datetime is None:
        from_datetime = utcnow()
    delta = target_datetime - from_datetime
    return delta.days


# def next_month(from_datetime, months=1):
#     return from_datetime + relativedelta.relativedelta(months=months)


def parse_date(date_str, fmt='%Y-%m-%d'):
    return datetime.strptime(date_str, fmt).date()

today = date.today





# -*- coding:utf-8 -*-
import sys
from importlib import import_module
from pathlib import Path


class AnanasConstant(object):

    docs = ""
    constant_docs = ""

    def __init__(self, **kwargs):
        """

        :param if_set:
        :param module_dir:
        :param module_name:
        :param title:
        """
        self.ROOT = kwargs["ROOT"]
        self.if_set = kwargs.get('if_set')
        self.module_dir = kwargs.get('module_dir')
        self.module_name = kwargs.get('module_name')
        self.title = kwargs["title"]
        self.title = "\n\n## " + self.title + "\n\n"
        self.path = kwargs.get('path')

    def set_md(self):
        """

        :return:
        """
        self.make_mk_str()
        self.write_file()

    def make_mk_str(self):
        """

        :return:
        """

        if not self.if_set:
            return

        # model_name = self.module_dir.replace(self.ROOT, "").lstrip("/").replace("/", ".") + "." + self.module_name.rstrip(".py")
        file = self.module_dir + "/" + self.module_name + ".py"
        try:
            with open(file, "r", encoding="utf-8") as p:
                content = p.read()
        except Exception as e:
            content = "生成失败：%s" % file.replace(self.ROOT, "")

        self.docs += "~~~python\n"
        self.docs += content + "\n"
        self.docs += "~~~\n"

        self.docs = self.title + self.docs

    def write_file(self):
        file_dir = self.path + "/docs/constant.md"
        f = open(file_dir, "w+", encoding='utf-8')
        f.write(self.docs)
        f.close()




# -*- coding:utf-8 -*-
import sys
from importlib import import_module
import os, glob, re
from datetime import datetime
from bson import ObjectId
from pathlib import Path

class AnanasDBTable(object):

    docs = ""
    constant_docs = ""

    def __init__(self, **kwargs):
        """

        :param if_set:
        :param module_dir:
        :param module_name:
        :param title:
        """
        self.ROOT = kwargs["ROOT"]
        self.if_set = kwargs.get('if_set')
        self.module_dir = kwargs.get('module_dir')
        self.module_name = kwargs.get('module_name')
        self.title = kwargs["title"]
        self.title = "\n\n## " + self.title + "\n\n"
        self.path = kwargs.get('path')
        self.kwargs = kwargs

    def set_md(self):
        """

        :return:
        """
        self.make_mk_str()
        self.write_file()

    def make_mk_str(self, schema_dir=None):
        """

        :return:
        """

        if not self.if_set:
            return

        file = schema_dir or os.path.join(self.module_dir, self.module_name)

        try:
            for schema_dir in glob.glob("%s/*" % file):
                schema_dir = Path(schema_dir).as_posix()
                if "__pycache__" in schema_dir:
                    continue

                objs_mp = {}
                content = ""

                if os.path.isdir(schema_dir):
                    self.make_mk_str(schema_dir)
                else:
                    model_name = schema_dir.replace(self.ROOT, "").lstrip("/").replace("/", ".").replace(".py", "")
                    objs = import_module(model_name)

                    for p in dir(objs):
                        if p.startswith("__"):
                            continue

                        if p == "BaseSchema":
                            continue

                        if type(getattr(objs, p)) != type:
                            continue

                        if not hasattr(getattr(objs, p), "collection"):
                            continue

                        if not getattr(getattr(objs, p), "collection"):
                            continue

                        # 字段：getattr(objs, "DemoSchema")()
                        # 类属性：getattr(objs, "DemoSchema")
                        objs_mp[p] = getattr(objs, p)

                    __, filename = os.path.split(schema_dir)
                    if not re.findall('[A-Za-z]\w+_schema\.py$', filename):
                        continue
                    try:
                        with open(schema_dir, "r", encoding="utf-8") as p:
                            find_class = False
                            schema_obj = None
                            fields = []
                            for line in p.readlines():
                                # 源码模式显示
                                # if find_class is False:
                                #     if re.findall(r'class\s+(.+?)Schema[(]BaseSchema[)]:', line):
                                #         find_class = True
                                #         content += line
                                # else:
                                #     if "@classmethod" in line or "@staticmethod" in line:
                                #         break
                                #     elif "__init__(self)" not in line and re.findall(r'def\s+(.+?)[(].*?[)]:', line):
                                #         break
                                #     else:
                                #         # if "def __init__(self):" in line:
                                #         #     content += line.replace("def __init__(self):", "数据库字段如下：")
                                #         # else:
                                #         #     content += line.replace("self.", "")
                                #         content += line.replace("self.", "")

                                result = re.findall(r'class\s+(.+?)[(]BaseSchema[)]:', line)
                                if find_class is False:
                                    if result:
                                        find_class = True
                                        schema_name = result[0]
                                        schema_obj = objs_mp[schema_name]
                                        content = self.get_table_name(schema_obj, content)
                                # 发现另外一个Schema，先把当前收集到的字段组装完成
                                elif result:
                                    content = self.get_field_data(schema_obj, fields, content)

                                    # 另一个表
                                    schema_name = result[0]
                                    schema_obj = objs_mp[schema_name]
                                    content = self.get_table_name(schema_obj, content)
                                    fields = []
                                    fields = self.get_fields(fields, line)
                                else:
                                    if "@classmethod" in line or "@staticmethod" in line:
                                        continue
                                    elif "__init__(self)" not in line and re.findall(r'def\s+(.+?)[(].*?[)]:', line):
                                        continue
                                    elif "self." not in line:
                                        continue
                                    else:
                                        fields = self.get_fields(fields, line)

                            content = self.get_field_data(schema_obj, fields, content)

                            self.docs += content
                    except Exception as e:
                        self.docs += "生成失败 %s:%s\n" % (schema_name, str(e))
        except Exception as e:
            self.docs += "生成失败：%s, %s\n" % (file, str(e))

        self.docs = self.title + self.docs

    def get_table_name(self, schema_obj, content):
        """
        获取表名
        :param result:
        :param objs_mp:
        :param content:
        :return:
        """
        collection = schema_obj.collection
        content += "###" + "表: " + collection + "\n"
        content += "说明:" + schema_obj.__doc__.lstrip("\n").rstrip(" ")
        return content

    def get_fields(self, fields, line):
        """
        获取字段
        :param fields:
        :param line:
        :return:
        """
        try:
            line = line.replace("\n", "")
            # 字段定义与描述
            result = re.findall(r'\s+self.(.+?)\s+=\s+(.*?)\s*(#.*?)$', line) or \
                     re.findall(r'\s+self.(.+?)\s+=\s+(.*?)\s*(#.*?)$', line) or \
                     re.findall(r'\s+self.(.+?)\s+=\s+(.*?)$', line)
            if result:
                fields.append(result[0])
        except Exception as e:
            pass
        finally:
            return fields

    def get_field_data(self, obj, fields, content):
        """
        组装指定表的字段结构
        :param class_name:
        :param fields:
        :return:
        """
        def _get_max_len():
            max_len = 0
            for item in fields:
                field_id = item[0]
                v = getattr(obj(), field_id)
                if ObjectId.is_valid(v):
                    v = len(str(v))
                elif isinstance(v, datetime):
                    v = 20
                elif isinstance(v, (dict, list, tuple)):
                    v = 10
                else:
                    v = len(str(v))

                if v > max_len:
                    max_len = v

            return max_len

        max_len = _get_max_len()

        content += "~~~python\n"
        for item in fields:
            if len(item) > 2:
                field_id = item[0]
                default_default = item[1]
                desc = item[2]
            else:
                field_id = item[0]
                default_default = item[1]
                desc = ""

            v = getattr(obj(), field_id)
            if ObjectId.is_valid(v):
                v = str(v)
            elif isinstance(v, datetime):
                v = "datatime日期格式"
            elif isinstance(v, dict):
                pass

            # 对于数字，当他是字符类型时，加双引号
            if type(v) in [int, float, bool]:
                content += '%s = %s' % (field_id, v)
                space_len = max_len - (len(str(v)) + len(field_id)) + 2
            elif type(v) in [tuple, list, dict]:
                content += '%s = %s' % (field_id, self.set_resp_md(v))
                if not v:
                    space_len = max_len - (0 + len(field_id))
                else:
                    space_len = max_len - 1 + 5
            else:
                content += '%s = "%s"' % (field_id, v)
                space_len = max_len - (len(str(v)) + len(field_id))

            content += ' ' * (space_len+10) + desc + "\n"

        content += "~~~\n"

        return content

    def set_resp_md(self, str_data):
        """

        :return:
        """
        str_v = ""
        self.space_seq = 1

        def get_space():
            return self.space_seq * "   "

        def get_type_fun(obj):
            t = type(obj)
            if t in [dict, list, tuple]:
                return {
                    dict: gen_dict_obj,
                    list: gen_list_obj,
                    tuple: gen_list_obj,
                }[t](obj)
            else:
                return get_string_obj(obj)

        def get_string_obj(obj):
            STR2TYPE = {"int": int,
                        "integer": int,
                        "string": str,
                        "str": str,
                        "unicode": "",
                        "float": float,
                        "bool": float,
                        "dict": dict,
                        "json": ""}
            # suffix = ""
            # for t, v in STR2TYPE.items():
            #     if "|%s" % t in str(obj):
            #         obj = obj.replace("|%s" % t, "")
            #         suffix = ", 类型%s" % t
            #
            # s = ""
            # s += str(obj) + suffix
            # return s + ",\n"

            if type(obj) in [str]:
                s = '"%s"' % obj
            elif type(obj) == bytes:
                s = '"%s"' % obj.decode("u8")
            else:
                s = str(obj)
            return s + ",\n"

        def gen_dict_obj(obj):
            self.space_seq += 1
            space = get_space()

            str = ""
            # str += "\n"
            str += "{\n"
            for k, n in obj.items():
                str += space + "  \"%s\"" % k + ": " + get_type_fun(n)
            str += space + "},\n"
            self.space_seq -= 1
            return str

        def gen_list_obj(obj):
            self.space_seq += 1
            space = get_space()

            str = ""
            # str += "\n"
            str += "[\n"
            for item in obj:
                # str += "\n"
                str += space + "   " + get_type_fun(item)
                # str += space + "   ...,\n"
                # break

            str += space + "],"
            str += "\n"
            self.space_seq -= 1
            return str

        if type(str_data) == dict:
            for key, value in str_data.items():
                str_v += get_space()
                str_v += "\"%s\"" % key + ": " + get_type_fun(value)
                self.space_seq = 1
        else:
            for value in str_data:
                str_v += get_space()
                str_v += get_type_fun(value)
                self.space_seq = 1

        if type(str_data) == dict:
            return "{\n" + str_v + "}" if str_data else "{" + str_v + "}"
        else:
            return "[\n" + str_v + "]" if str_data else "[" + str_v + "]"

    def write_file(self):
        file_dir = self.path + "/docs/tables.md"
        f = open(file_dir, "w+", encoding='utf-8')
        f.write(self.docs)
        f.close()




# -*- coding:utf-8 -*-
import sys, datetime
from lcyframe.libs import yaml2py

class AnanasTable(object):

    def __init__(self, **kwargs):
        self.path = kwargs.get('path')
        self.index_doc_config = kwargs.get('index_doc_config')

    def set_index(self):
        """

        :return:
        """
        title = self.index_doc_config.get('title')
        navs = self.index_doc_config.get('nav')
        content = ".. LcyFrame documentation\n\n"
        content += title + "\n" + len(title) * 2 * "=" + "\n\n"
        content += self.index_doc_config.get('content') + "\n\n"
        content += "目录: \n\n"
        content += ".. toctree::\n   :maxdepth: 2\n\n"

        if not navs:
            content += "   docs/" + "api" + ".md\n"
        else:
            for nav in navs:
                content += "   docs/" + nav + ".md\n"

        content += "\n\n"
        f = open(self.path + '/index.rst', "w+", encoding='utf-8')
        f.write(content)
        f.close()





# -*- coding:utf-8 -*-
import sys
from importlib import import_module



class AnanasError(object):

    docs = ""
    error_docs = ""

    def __init__(self, **kwargs):
        """

        :param if_set_errors:
        :param module_dir:
        :param module_name:
        :param error_title:
        """
        self.ROOT = kwargs["ROOT"]
        self.if_set_errors = kwargs.get('if_set_errors')
        self.module_dir = kwargs.get('module_dir')
        self.module_name = kwargs.get('module_name')
        self.error_title = kwargs.get('error_title', "错误码")
        self.title = "\n## " + kwargs.get('error_title', "错误码") + "\n\n"
        self.path = kwargs.get('path')

    def set_md(self):
        """

        :return:
        """
        self.set_errors()
        self.write_file()

    def set_errors(self):
        """

        :return:
        """
        # check if set errors
        if not self.if_set_errors:
            return

        model_name = self.module_dir.replace(self.ROOT, "").lstrip("/").replace("/", ".") + "." + self.module_name.rstrip(".py")
        errors = import_module(model_name)

        # set docs
        error_list = errors.__dict__
        data = {}
        code_len = code_name_len = message_len = zh_message_len = help_len = 0
        for key, error in error_list.items():
            obj_list = dir(error)
            if 'code' not in obj_list:
                continue
            code = str(error.code)
            if 'code_name' in obj_list:
                code_name = error.code_name
            else:
                code_name = ""
            if 'message' in obj_list:
                message = str(error.message)
            else:
                message = ""
            if 'zh_message' in obj_list:
                zh_message = str(error.zh_message)
            else:
                zh_message = ""
            if 'help' in obj_list:
                help = str(error.help)
            else:
                help = '' # "无说明,详询技术"

            if len(code) > code_len:
                code_len = len(code)
            if len(code_name) > code_name_len:
                code_name_len = len(code_name)
            if len(message) > message_len:
                message_len = len(message)
            if len(zh_message) > zh_message_len:
                zh_message_len = len(zh_message)

            zh_message = zh_message or message
            zh_message_len = zh_message_len or message_len
            if len(help) > help_len:
                help_len = len(help)
            data[code] = {
                "code": code,
                "code_name": code_name,
                "message": message,
                "zh_message": zh_message,
                "help": help
            }
        keys = data.keys()
        int_keys = list(map(int, keys))
        int_keys.sort()

        for k in int_keys:
            docs = ""
            error = data[str(k)]
            docs += "|" + error.get('code').ljust(code_len,  " ") + "|"
            docs += error.get('code_name').ljust(code_name_len,  " ") + "|"
            # docs += error.get('message').ljust(message_len,  " ") + "|"
            docs += error.get('zh_message').ljust(zh_message_len,  " ") + "|"
            docs += error.get('help').ljust(help_len,  " ") + "|"

            docs += "\n"
            self.docs += docs
        # self.title += "|错误码".ljust(code_len,  " ") + "|错误名称".ljust(code_name_len,  " ") + "|Message".ljust(message_len, " ") + "|说明".ljust(zh_message_len,  " ") + "|\n"
        # self.title += "|".ljust(code_len, "-") + "|".ljust(code_name_len, "-") + "|".ljust(message_len, "-") + "|".ljust(zh_message_len, "-") + "|\n"

        # 去掉英文message
        self.title += "|错误码".ljust(code_len, " ") + "|错误名称".ljust(code_name_len, " ") + "|说明".ljust(zh_message_len, " ") + "|解决办法".ljust(help_len, " ") +"|\n"
        self.title += "|".ljust(code_len, "-") + "|".ljust(code_name_len, "-") + "|".ljust(help_len, "-") + "|\n"

        self.docs = self.title + self.docs

    def write_file(self):
        file_dir = self.path + "/docs/error.md"
        f = open(file_dir, "w+", encoding='utf-8')
        f.write(self.docs)
        f.close()




# -*- coding: utf-8 -*-
"""
docs

"""
import os, io
from os import path
import sys
import shutil
from . import conf
from . import makefile
from lcyframe.libs import pytemplate
from lcyframe.libs import Copyright

package_dir = path.abspath(path.dirname(__file__))

try:
    # check if colorama is installed to support color on Windows
    import colorama
except ImportError:
    colorama = None

if False:
    # For type annotation
    from typing import Dict  # NOQA

codes = {}  # type: Dict[str, str]

def mkdir(path):
    """

    :param path:
    :return:
    """
    path = path.strip()
    path = path.rstrip("\\")

    is_exists = os.path.exists(path)
    if not is_exists:
        os.makedirs(path)
    else:
        pass


def color_terminal():
    # type: () -> bool
    if sys.platform == 'win32' and colorama is not None:
        colorama.init()
        return True
    if not hasattr(sys.stdout, 'isatty'):
        return False
    if not sys.stdout.isatty():
        return False
    if 'COLORTERM' in os.environ:
        return True
    term = os.environ.get('TERM', 'dumb').lower()
    if term in ('xterm', 'linux') or 'color' in term:
        return True
    return False

def nocolor():
    # type: () -> None
    if sys.platform == 'win32' and colorama is not None:
        colorama.deinit()
    codes.clear()


def main(project_name, path, api_schema_dir, errors_dir, constant_dir, model_dir, docs_agree_str):
    if not color_terminal():
        nocolor()
    error = "MakeError:[%s]"

    # make project dir
    try:
        if not path:
            print(error % "Please enter the project name")
            exit()
    except:
        print(error % "Please enter the project name")
        exit()

    is_exists = os.path.exists(path)
    if is_exists:
        print(error % "The directory already exists, do not duplicate it")
        exit()

    docs_path = path + "/docs"
    mkdir(path)
    mkdir(docs_path)

    with open(docs_path + "/api.md", "w", encoding='utf-8') as p:
        p.write("")

    with open(docs_path + "/agree.md", "w", encoding='utf-8') as w:
        w.write(docs_agree_str or pytemplate.get_docs_agree())

    with open(docs_path + "/error.md", "w", encoding='utf-8') as p:
        p.write("##三、错误码说明")

    # mkconf
    conf_server = conf.Config(path=path)
    conf_server.config_dict['project'] = project_name + "接口文档"
    conf_server.config_dict['copyright'] = "Document is automatic generation for %s. %s" % (Copyright.frame_name, Copyright.auto_email)
    conf_server.config_dict['author'] = Copyright.auto_name
    conf_server.mk_conf(api_schema_dir, errors_dir, constant_dir, model_dir)

    # load init files
    now_dir = path + "/template"
    shutil.copytree(package_dir + '/template', str(now_dir))
    shutil.copyfile(package_dir + '/index.rst', path + '/index.rst')
    mkfile_server = makefile.MakeFile(path=path)
    mkfile_server.set_make_file()
    # print("%s project created successfully!" % path)



if __name__ == "__main__":
    main(sys.argv[1], "", "project_name")



# -*- coding: utf-8 -*-

class AnanasDocError(Exception):

    def __init__(self, message=None, code=None, code_name=None, *args, **kwargs):
        super(AnanasDocError, self).__init__(*args, **kwargs)
        if message:
            self.message = message
        if code is not None:
            self.code = code
        if code_name is not None:
            self.code_name = code_name

class ErrorSetApi(AnanasDocError):

    code = 10001
    code_name = 'Error_Set_Api'
    # message = 'set api error'



# -*- coding:utf-8 -*-
import os
import sys
import logging
from . import error
from . import log
from . import table
from . import rest_api
from . import constant
from . import db_table
import platform

from importlib import import_module


class AnanasTemplate(object):

    def __init__(self, app, path):
        self.app = app
        self.path = path

    def run(self, executable):
        """

        :return:
        """
        config = import_module(self.path.replace(self.app.ROOT, "").lstrip("/").replace("/", ".") + ".conf")
        errors_doc_config = config.errors_doc_config
        log_doc_config = config.log_doc_config
        index_doc_config = config.index_doc_config
        api_doc_config = config.api_doc_config
        constant_doc_config = config.constant_doc_config
        table_doc_config = config.table_doc_config

        # make version
        if log_doc_config.get('if_set_log') is True:
            os.system("vim " + path + "/template/_version/.note")
            os.system("vim " + path + "/template/_version/.version")

        # make markdown
        self.make_markdown(errors_doc_config=errors_doc_config,
                           log_doc_config=log_doc_config,
                           index_doc_config=index_doc_config,
                           api_doc_config=api_doc_config,
                           constant_doc_config=constant_doc_config,
                           table_doc_config=table_doc_config)
        # remove cache
        try:
            os.remove("rm -rf " + self.path + "/build")
        except:
            pass

        # make html
        if self.app.api_docs.get("stdout", False) is False:
            if platform.system() == "Windows":
                stdout = "> NUL"
            else:
                stdout = "> /dev/null 2>&1"
        else:
            stdout = ""
        os.system("%s -msphinx -M html " % executable + self.path + " " + self.path + "/build %s" % stdout)
        logging.warning('Docs Running on http://%s:%s/docs' % (self.app.uri, self.app.port))

    def make_markdown(self, **kwargs):
        """

        :param kwargs:
        :return:
        """
        # load config
        errors_doc_config = kwargs.get('errors_doc_config')
        log_doc_config = kwargs.get('log_doc_config')
        index_doc_config = kwargs.get('index_doc_config')
        api_doc_config = kwargs.get('api_doc_config')
        constant_doc_config = kwargs.get('constant_doc_config')
        table_doc_config = kwargs.get('table_doc_config')

        # make api md
        if api_doc_config.get("if_set_api") is True:
            api_server = rest_api.AnanasRestApi(path=self.path, api_doc_config=api_doc_config)
            api_server.set_api()
            # print("Api document was generated successfully！")

        # make error md
        if errors_doc_config.get('if_set_errors') is True:
            error_server = error.AnanasError(ROOT=self.app.ROOT, path=self.path, **errors_doc_config)
            error_server.set_md()
            # print("Error document was generated successfully！")

        # make constant md
        if constant_doc_config.get('if_set') is True:
            constant_server = constant.AnanasConstant(ROOT=self.app.ROOT, path=self.path, **constant_doc_config)
            constant_server.set_md()

        # make table md
        if table_doc_config.get('if_set') is True:
            table_server = db_table.AnanasDBTable(ROOT=self.app.ROOT, path=self.path, **table_doc_config)
            table_server.set_md()

        # make log md
        if log_doc_config.get('if_set_log') is True:
            log_server = log.AnanasLog(path=self.path,
                                       log_doc_config=log_doc_config)
            log_server.set_md()
            # print("Log document was generated successfully！")

        # make index
        table_server = table.AnanasTable(path=self.path,
                                         index_doc_config=index_doc_config)
        table_server.set_index()
        # print("Index document was generated successfully！")

        # init log
        if log_doc_config.get('if_set_log') is True:
            log_server.initialize()

if __name__ == "__main__":
    path = sys.argv[1]
    AnanasTemplate(path).run(sys.executable)



# -*- coding:utf-8 -*-
import sys, datetime
from lcyframe.libs import yaml2py

class AnanasLog(object):

    note_cache_file = "template/_version/.note"

    version_cache_file = "template/_version/.version"

    version_yml = "template/_version/version.yml"

    log_md = "docs/log.md"

    note_list = []

    def __init__(self, **kwargs):
        self.path = kwargs.get('path')
        self.log_doc_config = kwargs.get('log_doc_config')
        self.note_cache_file = self.path + "/" + self.note_cache_file
        self.version_cache_file = self.path + "/" + self.version_cache_file
        self.version_yml = self.path + "/" + self.version_yml
        self.log_md = self.path + "/" + self.log_md

    def load_note(self):
        """

        :return:
        """
        note = ""
        f = open(self.note_cache_file, "r", encoding='utf-8')
        while True:
            line = f.readline()
            if not line:
                break
            if line[0] == "#":
                continue
            note += "* " + str(line)
            self.note_list.append(line)
        f.close()
        self.note = note

    def load_version(self):
        """

        :return:
        """
        f = open(self.version_cache_file, "r", encoding='utf-8')
        while True:
            line = f.readline()
            line.strip()
            if not line:
                break
            if line[0] == "#":
                continue
            self.version = line.strip()
            break
        f.close()

    def set_md(self):
        """

        :return:
        """
        self.load_version()
        self.load_note()
        now_date = datetime.datetime.now().strftime('%Y-%m-%d')
        now_time = datetime.datetime.now().strftime('%H:%M')
        yml_file = open(self.version_yml, 'r', encoding='utf-8')
        log = yaml2py.load(yml_file).get('version', None)
        docs = {
            'date': now_date,
            'time': now_time,
            'version': self.version,
            'author': self.log_doc_config.get('author', ''),
            'note': self.note_list
        }
        if log:
            new_v = log[0]
            if new_v['version'] == self.version:
                log[0] = docs
            else:
                log.insert(0, docs)
        else:
            log = [docs]
        _doc = {
            'version': log
        }
        yml_file.close()
        yml_file = open(self.version_yml, 'w', encoding='utf-8')
        yaml2py.dump(_doc, yml_file)
        yml_file.close()
        log_doc = "\n## "
        log_doc += self.log_doc_config.get('log_title') + "\n"
        for _log in log:
            log_doc += "### " + _log['date'] + "\n\n"
            log_doc += "**版本:** " + _log['version'] + "\n\n"
            log_doc += "**作者:** " + _log['author'] + "\n\n"
            log_doc += "**更新于:** " + _log['date'] + " " + _log['time'] + "\n\n"
            log_doc += "**更新内容:** \n\n"
            for v in _log['note']:
                log_doc += "* " + v
            log_doc += "\n"

        f = open(self.log_md, "w+", encoding='utf-8')
        f.write(log_doc)
        f.close()

    def initialize(self):
        """

        :return:
        """
        note_doc = """
# Please enter a submission message to explain the updated content.
#
# Lines starting with '#' will be ignored, and each line represents a title
        """
        note_doc += "\n"
        f = open(self.note_cache_file, "w+", encoding='utf-8')
        f.write(note_doc)
        f.close()
        self.load_version()
        version_doc = """# Write a message for version:
        """
        version_doc += "\n# The latest version is " + str(self.version)
        version_doc += "\n# Lines starting with '#' will be ignored.\n"
        f = open(self.version_cache_file, "w+", encoding='utf-8')
        f.write(version_doc)
        f.close()



# -*- coding: utf-8 -*-
from lcyframe.libs import Copyright

class Config(object):
    config_dict = dict(
        extensions = [
            'sphinx.ext.autodoc',
            'sphinx.ext.doctest',
            'sphinx.ext.intersphinx',
            'sphinx.ext.todo',
            'sphinx.ext.coverage',
            'm2r'
        ],
        templates_path = ['_templates'],
        source_suffix = ['.rst', '.md'],
        master_doc = 'index',
        exclude_patterns = [],
        pygments_style = 'sphinx',
        todo_include_todos = True,
        latex_elements = {},
        html_sidebars = {
            '**': [
                'about.html',
                'navigation.html',
                'relations.html',
                'searchbox.html',
                'donate.html',
            ]
        },
        project = u'LcyFrame',
        copyright = u'2017, Lcylln',
        author = u'Lcylln',
        version = u'',
        release = u'',
        language = 'zh_CN',
        html_theme = 'sphinx_rtd_theme',
        html_static_path = ['template/_static'],
        index_doc_config = {
            'title': "LcyFrame",
            'content': "LcyFrame",
            "nav": []
        },
        errors_doc_config = {
            "if_set_errors": False,
            "module_dir": "/path",
            "module_name": "errors",
            "error_title": "",
        },
        log_doc_config = {
            "if_set_log": False,
            "author": "Lcylln",
            "log_title": "",
        }
    )

    def __init__(self, **kwargs):
        self.path = kwargs.get('path')

    def mk_conf(self, api_schema_dir, errors_dir, constant_dir, table_schema_dir):
        """

        :return:
        """
        conf_str = ""
        conf_str += """# -*- coding: utf-8 -*-
# docs config
#
import sphinx_rtd_theme

extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.doctest',
    'sphinx.ext.intersphinx',
    'sphinx.ext.todo',
    'sphinx.ext.coverage',
    'm2r'
]
templates_path = ['_templates']
source_suffix = ['.rst', '.md']
master_doc = 'index'
exclude_patterns = []
pygments_style = 'sphinx'
todo_include_todos = True
latex_elements = {}
html_sidebars = {
    '**': [
        'about.html',
        'navigation.html',
        'relations.html',  # needs 'show_related': True theme option to display
        'searchbox.html',
        'donate.html',
    ]
}

# set document title author
"""
        conf_str += "\n"
        conf_str += "project = u'" + self.config_dict['project'] + "'"
        conf_str += "\n"
        conf_str += "copyright = u'" + self.config_dict['copyright'] + "'"
        conf_str += "\n"
        conf_str += "author = u'" + self.config_dict['author'] + "'"
        conf_str += "\n"
        conf_str += """
# API version
version = u''
release = u''

# language
language = 'zh_CN'

# theme
html_theme = 'sphinx_rtd_theme'
html_theme_path = [sphinx_rtd_theme . get_html_theme_path()]

# static path
html_static_path = ['template/_static']

# index
index_doc_config = {
    'title': "欢迎使用!",
    'content': "Skype：lcylln001@gmail.com",
    "nav": ["agree", "api", "error", "constant", "tables"]
}

# api config
api_doc_config = {
    "if_set_api": True,            # Using YML to generate .md
    "api_schema_dir": "%s",               # abs path of your api_schema dir. like this:xxx/xxx/api_schema/
    "leve": 2,                     #
    "title": "二、接口说明",         # give the title by level start path.
    "schema_template": "lcyframe"  # with yml template。 lcyframe, default
}

# error code config
errors_doc_config = {
    "if_set_errors": True,
    "module_dir": "%s",            # errors.py path, default like '~/www/demo/utils'
    "module_name": "errors",
    "error_title": "三、错误码",
}

# constant config
constant_doc_config = {
    "if_set": True,
    "module_dir": "%s",           # constant.py path, default like '~/www/demo/utils'
    "module_name": "constant",
    "title": "四、常量值",
}

# db table 
table_doc_config = {
    "if_set": True,
    "module_dir": "%s",           # xxx_schema.py path, default like '~/model/schema'
    "module_name": "schema",
    "title": "五、数据库设计",
}

# doc log
log_doc_config = {
    "if_set_log": False,
    "author": "%s",
    "log_title": "五、更新日志",
} 
        """ % (api_schema_dir, errors_dir, constant_dir, table_schema_dir, Copyright.auto_name)

        f = open(self.path + "/conf.py", "w+", encoding='utf-8')
        f.write(conf_str)
        f.close()




# coding=utf-8
import logging
import os
import sys
from lcyframe.libs.docs import start
from .template import AnanasTemplate
from threading import Thread
from traceback import format_exc
import time
from ..utils import random_string, fix_path
import shutil
import platform
from importlib import import_module

class GenerateDocs(object):

    @classmethod
    def start(cls, app):
        """
        Auto generate the docs for this project.
        :return:
        """
        if not app.kwargs.get("api_docs"):
            return

        if os.path.exists(os.path.join(app.ROOT, "conf")):
            raise Exception("Can not exists file 'conf' dir in you project. It is reserved for the docs directory.")

        if not app.api_docs.get("auto_generate"):
            return

        t = Thread(target=cls.do, args=(app, ))
        t.start()

    @classmethod
    def do(cls, app):
        new_docs_name = "%s_%s" % (random_string(3), random_string(3))
        new_docs_dir = os.path.dirname(app.docs_dir) + "/docs_%s" % new_docs_name
        docs_agree_str = cls.get_docs_agree(app)
        try:
            import signal
            # logging.info("generate docs...")
            if not os.path.exists(new_docs_dir):
                start.main(app.kwargs.get("project_name"),
                           new_docs_dir,
                           app.api_schema_dir,
                           app.errors_dir,
                           app.constant_dir,
                           app.model_dir,
                           docs_agree_str)

            AnanasTemplate(app, new_docs_dir).run(sys.executable)
            cls.change_docs(app, new_docs_dir)
            # logging.info("generate docs done!")
        except Exception as e:
            logging.warning(format_exc())
            # logging.warning("Api docs generate Faile! you can do it yourself： cd 'docs_dir' && make html")
            cls.remove_docs(new_docs_dir)

    @classmethod
    def get_docs_agree(cls, app):
        """

        :param app:
        :return:
        """
        try:
            api_docs = app.get_path(app.api_docs.get("agree_dir", "context"))
            # model_name = api_docs.replace(app.ROOT, "").lstrip("/").replace("/", ".") + ".docs_agree"
            # docs_agree = import_module(model_name)
            file = api_docs + "/docs_agree.md"
            if os.path.exists(file):
                with open(api_docs + "/docs_agree.md", "r") as p:
                    return "".join(p.readlines())
            else:
                return ""
        except Exception as e:
            logging.error("get docs_agree config error:" + str(e))
            return ""

    @classmethod
    def change_docs(cls, app, new_docs_dir):
        cls.remove_docs(app.docs_dir)
        if app.api_docs.get("output", "html") == "html":
            shutil.copytree(fix_path(os.path.join(new_docs_dir, "build/html")), app.docs_dir)
        else:
            shutil.copytree(new_docs_dir, app.docs_dir)
        cls.remove_docs(new_docs_dir)

    @classmethod
    def remove_docs(cls, docs_dir):
        if os.path.exists(docs_dir):
            shutil.rmtree(docs_dir)
